﻿using System;
using System.Data.OleDb;
using System.Windows.Forms;

using DevExpress.Utils;
using DevExpress.Utils.Menu;
using DevExpress.XtraEditors;
using DevExpress.XtraGrid.Menu;
using DevExpress.XtraGrid.Views.Grid;
using DevExpress.XtraGrid.Views.Grid.ViewInfo;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 연결
        /// </summary>
        private OleDbConnection connection;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.connection = new OleDbConnection("Provider=Microsoft.Jet.OLEDB.4.0;Data Source=nwind.mdb");

            this.gridView.InitializeView
            (
                true,                           // Multi Select
                GridMultiSelectMode.CellSelect, // Grid Multi Select Mode
                DrawFocusRectStyle.CellFocus,   // Draw Focus Rect Style
                true,                           // Show Indicator
                true,                           // Show Column Headers
                true,                           // Allow Column Moving
                true,                           // Allow Column Resizing
                true,                           // Allow Filter
                true,                           // Allow Sort
                false,                          // Allow Cell Merge
                EditorShowMode.Default,         // Editor Show Mode
                false                           // Editable
            );

            this.gridView.OptionsBehavior.AutoPopulateColumns = true;

            this.gridControl.DataSource = Product.GetProductList(this.connection);

            this.gridView.BestFitColumns();

            this.gridView.MouseDown += gridView_MouseDown;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 그리드 뷰 마우스 DOWN 처리하기 - gridView_MouseDown(sender, e)

        /// <summary>
        /// 그리드 뷰 마우스 DOWN 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gridView_MouseDown(object sender, MouseEventArgs e)
        {
            GridView gridView = sender as GridView;
            
            GridHitInfo gridHitInfo = gridView.CalcHitInfo(e.Location);

            if(gridHitInfo.HitTest != GridHitTest.ColumnButton)
            {
                return;
            }

            ViewMenu viewMenu = new ViewMenu(gridView);

            viewMenu.Items.Add(GetMenuCheckItem
            (
                gridView,
                "Column &Auto Width",
                new EventHandler(columnAutoWidthMenuCheckItem_CheckedChanged))
            );

            viewMenu.Show(gridHitInfo.HitPoint);
        }

        #endregion
        #region 컬럼 자동 너비 메뉴 체크 항목 체크 변경시 처리하기 - columnAutoWidthMenuCheckItem_CheckedChanged(sender, e)

        /// <summary>
        /// 컬럼 자동 너비 메뉴 체크 항목 체크 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void columnAutoWidthMenuCheckItem_CheckedChanged(object sender, EventArgs e)
        {
            DXMenuCheckItem menuCheckItem = sender as DXMenuCheckItem;

            GridView gridView = menuCheckItem.Tag as GridView;

            if(gridView == null)
            {
                return;
            }

            gridView.OptionsView.ColumnAutoWidth = menuCheckItem.Checked;

            if(!gridView.OptionsView.ColumnAutoWidth)
            {
                gridView.BestFitColumns();
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 메뉴 체크 항목 구하기 - GetMenuCheckItem(gridView, caption, checkedChangedEventHandler)

        /// <summary>
        /// 메뉴 체크 항목 구하기
        /// </summary>
        /// <param name="gridView">그리드 뷰</param>
        /// <param name="caption">제목</param>
        /// <param name="checkedChangedEventHandler">체크 변경시 이벤트 핸들러</param>
        /// <returns>메뉴 체크 항목</returns>
        private DXMenuCheckItem GetMenuCheckItem(GridView gridView, string caption, EventHandler checkedChangedEventHandler)
        {
            DXMenuCheckItem menuCheckItem = new DXMenuCheckItem
            (
                caption,
                gridView.OptionsView.ColumnAutoWidth,
                null,
                checkedChangedEventHandler
            );

            menuCheckItem.Tag = gridView;

            return menuCheckItem;
        }

        #endregion
    }
}